import 'dart:io';

import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path_provider/path_provider.dart';

class BGHelperDB {
  static final _databaseName = "rf_config.db"; //rf_config.db
  static final _databaseVersion = 1;

  static final table = 'Config';

  static final jobId = '_id'; //_id
  static final jobName = 'JobName'; //JobName - name
  static final jobCompleteStatus = 'CompleteStatus'; //CompleteStatus - age

  // make this a singleton class
  BGHelperDB._privateConstructor();

  static final BGHelperDB instance = BGHelperDB._privateConstructor();

  // only have a single app-wide reference to the database
  static Database _database;
  Future<Database> get database async {
    if (_database != null) return _database;
    // lazily instantiate the db the first time it is accessed
    _database = await _initDatabase();
    return _database;
  }

  // this opens the database (and creates it if it doesn't exist)
  _initDatabase() async {
    Directory documentsDirectory = await getApplicationDocumentsDirectory();
    String path = join(documentsDirectory.path, _databaseName);
    return await openDatabase(path, version: _databaseVersion, onCreate: _onCreate);
  }

  // SQL code to create the database table
  Future _onCreate(Database db, int version) async {
    await db.execute('''
          CREATE TABLE $table (
            $jobId INTEGER PRIMARY KEY,
            $jobName TEXT NOT NULL,
            $jobCompleteStatus INTEGER NOT NULL
          )
          ''');
    await db.execute('''
          CREATE TABLE FetchHistory(RecordId INTEGER PRIMARY KEY AUTOINCREMENT, 
          TableName TEXT, Pages TEXT, TotalRecord INTEGER, Success INTEGER)
          ''');
  }

  // Helper methods

  // Inserts a row in the database where each key in the Map is a column name
  // and the value is the column value. The return value is the id of the
  // inserted row.
  Future<int> insert(Map<String, dynamic> row) async {
    Database db = await instance.database;
    return await db.insert(table, row);
  }

  // All of the rows are returned as a list of maps, where each map is
  // a key-value list of columns.
  Future<List<Map<String, dynamic>>> queryAllRows() async {
    Database db = await instance.database;
    return await db.query(table);
  }

  Future<List<Map<String, dynamic>>> get(String sql, List<dynamic> values) async {
    Database db = await instance.database;
    var ret = await db.rawQuery(sql, values);
    return ret; //await db.query(table);
  }

  // All of the methods (insert, query, update, delete) can also be done using
  // raw SQL commands. This method uses a raw query to give the row count.
  Future<int> queryRowCount() async {
    Database db = await instance.database;
    return Sqflite.firstIntValue(await db.rawQuery('SELECT COUNT(*) FROM $table'));
  }

  // We are assuming here that the id column in the map is set. The other
  // column values will be used to update the row.
  Future<int> update(String jobName, int status) async {
    Database db = await instance.database;
    return await db.rawUpdate('update Config set CompleteStatus = $status where JobName=?', [jobName]);
  }

  Future<int> updateFetchHistoryPages(String tableName, String pages, int totalRecord, int success) async {
    Database db = await instance.database;
    return await db.rawUpdate(
        'update FetchHistory set Pages = Pages || "$pages,", '
        'TotalRecord = $totalRecord, Success = $success where TableName=?',
        [tableName]);
  }

  Future<int> updateFetchHistorySuccess(String tableName, int success) async {
    Database db = await instance.database;
    return await db.rawUpdate('update FetchHistory set Success = $success where TableName=?', [tableName]);
  }

  Future<int> insertFetchHistory(String tableName, String pages, int totalRecord, int success) async {
    Database db = await instance.database;
    return await db.rawInsert('insert into FetchHistory(TableName, Pages, TotalRecord, Success) '
        'values("$tableName", "$pages" , $totalRecord, $success)');
  }

  // Deletes the row specified by the id. The number of affected rows is
  // returned. This should be 1 as long as the row exists.
  Future<int> delete(int id) async {
    Database db = await instance.database;
    return await db.delete(table, where: '$jobId = ?', whereArgs: [id]);
  }

  Future<int> deleteService(String serviceName) async {
    Database db = await instance.database;
    return await db.rawDelete('delete from Config where JobName=?', [serviceName]);
  }

  Future<int> clearFetchHistory(String model) async {
    Database db = await instance.database;
    return await db.rawDelete('delete from FetchHistory where TableName=?', [model]);
  }

  readFetchHistory(String model) async {
    Database db = await instance.database;
    var rec = await db.rawQuery('select * from FetchHistory where TableName=?', [model]);
    if (rec.length > 0) {
      return FetchHistory(
        rec.first['RecordId'],
        rec.first['TableName'],
        rec.first['Pages'],
        rec.first['TotalRecord'],
        rec.first['Success'],
      );
    } else {
      return null;
    }
  }
}

class FetchHistory {
  int RecordId;
  String TableName;
  String Pages;
  int TotalRecord;
  int Success;
  FetchHistory(this.RecordId, this.TableName, this.Pages, this.TotalRecord, this.Success);
}
