import 'package:rapidfire/Dynamic.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:sqflite/sqflite.dart';
import 'dart:io' as io;
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'Annotation.dart' as annotation;
import 'BGHelperDB.dart';
import 'Config.dart';
import 'GlobalScopeRF.dart';
import 'UtilRF.dart';

class Db {
  static Database _database;

  Db([Object x, bool reset = false]);

  Future<Database> get database async {
    if (_database != null) return _database;
    _database = await initDatabase();
    return _database;
  }

  initDatabase() async {
    io.Directory documentDirectory = await getApplicationDocumentsDirectory();
    String path = join(documentDirectory.path, 'student.db');
    var db = await openDatabase(path, version: 1, onCreate: _onCreate);
    return db;
  }

  _onCreate(Database db, int version) {
    createTable().forEach((element) async {
      await db.execute(element);
      print(element);
    });
  }

  execute(String sql) async {
    var db = await database;
    var result = await db.rawUpdate(sql);
    return result;
  }

  get(Object model, String sql, [List<dynamic> values, Function onSuccess]) async {
    List<Object> modelList = List();
    var db = await database;
    List<Map> result = await db.rawQuery(sql, values);
    result.forEach((element) {
      var newModel = Dynamic(model).newClass();

      element.forEach((key, value) {
        Dynamic(newModel).set(key, value);
      });
      modelList.add(newModel);
    });
    if (onSuccess != null) onSuccess(modelList);
    return modelList;
  }

  Future<Object> save(Object model) async {
    var pk = Dynamic(model).annotationData(annotation.Key);
    var value = Dynamic(model).get(pk.first['field']);
    if (value == null)
      return await add(model);
    else
      return await update(model, '${pk.first['field']} = ?', [value]);
  }

  Future<Object> add(Object model) async {
    var db = await database;

    Dynamic dyn = Dynamic(model);
    var data = dyn.modelToDataMap();
    var id = await db.insert(model.runtimeType.toString(), data);
    Dynamic(model).set(getPrimaryKeyName(model), id);
    return model;
  }

  Future<int> update(Object model, String where, List<dynamic> args) async {
    var data = Dynamic(model).modelToDataMap();
    var db = await database;
    return await db.update(
      model.runtimeType.toString(),
      data,
      where: where,
      whereArgs: args,
    );
  }

  Future<int> delete(Object model, int id, [List<Function> validators]) async {
    validators = validators ?? Libs.config.model.validators;
    var isValidValidForDeletion = true;
    var db = await database;
    var pk = Dynamic(model).annotationData(annotation.Key);
    validators?.forEach((element) {
      isValidValidForDeletion = element.call();
      if (!isValidValidForDeletion) return;
    });
    if (isValidValidForDeletion)
      return await db.delete(
        'student',
        where: '${pk.first['field']} = ?',
        whereArgs: [id],
      );
    else
      return Future.value(0);
  }

  Future close() async {
    var db = await database;
    db.close();
  }

  List<String> createTable() {
    List<String> createSql = new List();
    var pkSql = "";
    var pkFieldName = "";
    var pkFieldType = "";
    var pkFieldAuto = "";
    var models = Libs.appModels.get(); // New Model Style

    models.forEach((k, v) {
      var _annotation = Dynamic((v as ModelRF).model).annotationData(annotation.Key).first;

      if (_annotation != null) {
        pkFieldName = _annotation["field"];
        pkFieldType = _annotation["type"].toString();
        pkFieldAuto = _annotation["data"]["auto"] ? "AUTOINCREMENT" : "";
        pkSql = "${pkFieldName} ${convertSqlFieldType(pkFieldType)} PRIMARY KEY ${pkFieldAuto}";
      }

      var sql = "";
      Dynamic((v as ModelRF).model).modelMembers.forEach((name, type) {
        if (pkFieldName != name) {
          if (!type.toString().contains('List')) {
            sql = sql + name + " " + convertSqlFieldType(type.toString()) + ", ";
          }
        }
      });
      sql = sql.trim().substring(0, sql.length - 2);
      createSql.add("CREATE TABLE ${k}(${pkSql},${sql});");
    });
    return createSql;
  }

  String convertSqlFieldType(String s) {
    String type = "";
    if (s.contains("String")) {
      type = " TEXT";
    } else if (s.contains("long")) {
      type = " INTEGER";
    } else if (s.contains("byte[]")) {
      type = " BLOB";
    } else if (s.contains("int")) {
      type = " INTEGER";
    } else {
      type = "";
    }
    return type;
  }

  getPrimaryKeyName(Object model) {
    var pk = Dynamic(model).annotationData(annotation.Key);
    return pk.first['field'];
  }

  getPrimaryKeyValue(Object model) {
    var _model = Dynamic(model);
    var pk = _model.annotationData(annotation.Key);
    var pkName = pk.first['field'];
    var pkValue = _model.get(pkName.toString());
    return pkValue;
  }
}

//Type typeOfElementsInList<T>(List<T> e) => T;

enum DataSourceType { local, remote, memory }
enum DataSourceReadType { auto, none, quick, advanced }

class DataSourceRF<T> {
  int currentPage = 1;

  dynamic model;
  String sql = '';
  String initSql = '';
  List<T> itemList; // No Use ????

  DataSourceType type;
  Function stateUpdateFn;
  Function readListItemFn;
  Function deleteListItemFn;
  Function updateLoadingStateFn;
  bool isLoading = false;

  DataSourceRF({this.itemList, this.sql, this.type = DataSourceType.local}) {
    var modelType = typeOfElementsInList(itemList);
    this.model = modelType;
    var ps = Libs.config.data.pageSize;
    this.initSql = 'SELECT * FROM ${modelType.toString()} ORDER BY 1 DESC LIMIT ${ps} OFFSET 0';
    updateSql(0, 'desc');
  }

  String selectFields = '*';
  String whereFields = '';

  updateSql(int pageNo, String sortOrder) {
    var ps = Libs.config.data.pageSize;
    this.sql =
        'select ${selectFields} from ${this.model} ${whereFields.isEmpty ? '' : ' where '} ${whereFields} '
        'order by 1 ${sortOrder} limit ${ps} offset ${ps * pageNo}';
  }

  read(String sqlParam, int pageNo, DataSourceReadType readType) async {
    if (!isLoading) {
      isLoading = true;
      if (updateLoadingStateFn != null) updateLoadingStateFn(true);
      var data;
      Api api = Api();

      var model = typeOfElementsInList(itemList);

      processWhereOr(String clause, value) {
        String where = "";
        List<String> clauses = clause.split(',');
        clauses.forEach((element) {
          where += ' $element like "%$value%" or ';
        });
        where = where.substring(0, where.length - 3);
        return where;
      }

      if (readType == DataSourceReadType.none) {
        updateSql(pageNo, 'desc');
      } else if (readType == DataSourceReadType.auto) {
        updateSql(pageNo, 'desc');
      } else if (readType == DataSourceReadType.quick) {
        var instructions = sqlParam.split('#');
        var modelName = this.model;
        var _whereFields = instructions[0];
        var queryValue = instructions[1];
        var sortType = instructions[2];
        this.whereFields = processWhereOr(_whereFields, queryValue);
        updateSql(pageNo, 'desc');
        //'select * from ${modelName} where ${processWhereOr(whereFields, queryValue)} order by 1 ${sortType}';
      }

      if (type == DataSourceType.local) {
        data = await Db(null).get(model, readType == DataSourceReadType.none ? this.initSql : this.sql);
      } else {
        //data = await api.fetchRND(model, pageNo: pageNo);
        await api.remoteGet(model, pageNo, (_, __, rec) {
          data = rec;
        });
      }
      //sleep(Duration(seconds: 5));

      if (stateUpdateFn != null) stateUpdateFn(data, pageNo);
      isLoading = false;
      if (updateLoadingStateFn != null) updateLoadingStateFn(false);
      PageRF.dataSource = null;
    }
  }

  refresh() {
    read(sql, 0, DataSourceReadType.none);
  }

  delete(int index) {
    if (deleteListItemFn != null) deleteListItemFn(index);
  }

  getList() {
    if (readListItemFn != null) return readListItemFn();
  }
}
